Package: verifyr2
Type: Package
Title: Compare and Verify File Contents
Description: Extendable 'R6' file comparison classes, including a 'shiny' app for combining the comparison functionality into a file comparison application. The package idea originates from pharma companies' drug development processes, where statisticians and statistical programmers need to review and compare different versions of the same outputs and datasets. The package implementation itself is not tied to any specific industry and can be used in any context for easy file comparisons between different file version sets.
Version: 1.2.0
License: MIT + file LICENSE
Authors@R: c(
    person("Juha", "Javanainen", email = "ejuhjav@gmail.com", role = c("aut", "cre")),
    person("Tiina", "Kirsilä", email = "tiina.kirsila@gmail.com", role = "ctb"),
    person("Anna", "Wiksten", email = "a.s.wiksten@gmail.com", role = "ctb"))
Depends: R (>= 4.1.0)
Imports: base64enc, diffobj, dplyr, jsonlite, magrittr, mime, R6,
        rappdirs, shiny, stringr, striprtf, tibble
Suggests: DT, fs, magick, pkgload, pdftools, purrr, rmarkdown, shinyjs,
        shinyFiles, shinytest2, testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
Collate: 'FileComparator.R' 'BinaryFileComparator.R' 'Config.R'
        'Debugger.R' 'TxtFileComparator.R' 'FileComparatorFactory.R'
        'ImgFileComparator.R' 'PdfFileComparator.R'
        'TxtWithImagesComparator.R' 'RtfFileComparator.R'
        'list_files.R' 'list_folder_files.R' 'run_example.R'
        'utils-pipe.R'
NeedsCompilation: no
Packaged: 2026-05-23 10:48:38 UTC; juha
Author: Juha Javanainen [aut, cre],
  Tiina Kirsilä [ctb],
  Anna Wiksten [ctb]
Maintainer: Juha Javanainen <ejuhjav@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-23 11:30:02 UTC
Built: R 4.6.0; ; 2026-05-23 15:11:15 UTC; unix
