# Generated by roxygen2: do not edit by hand

export("%>%")
export(BinaryFileComparator)
export(Config)
export(Debugger)
export(FileComparator)
export(ImgFileComparator)
export(PdfFileComparator)
export(RtfFileComparator)
export(TxtFileComparator)
export(XlsxFileComparator)
export(create_comparator)
export(list_files)
export(list_folder_files)
export(run_example)
import(base64enc)
import(htmltools)
import(jsonlite)
import(rappdirs)
import(stringr)
import(striprtf)
importFrom(R6,R6Class)
importFrom(magrittr,"%>%")
