% Generated by roxygen2: do not edit by hand % Please edit documentation in R/PdfFileComparator.R \name{PdfFileComparator} \alias{PdfFileComparator} \title{PdfFileComparator.R} \description{ PdfFileComparator.R PdfFileComparator.R } \details{ Specialiced comparator for PDF file comparison. This comparator contains the custom handling for handling only PDF content part for the comparison. } \examples{ # The normal way for creating a comparator would be to call the generic # factory method verifyr2::create_comparator that will automatically create # the correct comparator instance based on the file types. file1 <- 'my_file1.pdf' file2 <- 'my_file2.pdf' comparator <- verifyr2::create_comparator(file1, file2) # If needed, an explicit comparator can be created as well. file1 <- 'my_file1.pdf' file2 <- 'my_file2.pdf' comparator <- PdfFileComparator$new(file1, file2) } \section{Super classes}{ \code{\link[verifyr2:FileComparator]{verifyr2::FileComparator}} -> \code{\link[verifyr2:BinaryFileComparator]{verifyr2::BinaryFileComparator}} -> \code{\link[verifyr2:TxtFileComparator]{verifyr2::TxtFileComparator}} -> \code{PdfFileComparator} } \section{Methods}{ \subsection{Public methods}{ \itemize{ \item \href{#method-PdfFileComparator-vrf_contents}{\code{PdfFileComparator$vrf_contents()}} \item \href{#method-PdfFileComparator-vrf_details_supported}{\code{PdfFileComparator$vrf_details_supported()}} \item \href{#method-PdfFileComparator-clone}{\code{PdfFileComparator$clone()}} } } \if{html}{\out{
Inherited methods

- `verifyr2::FileComparator$initialize()`
- `verifyr2::FileComparator$vrf_add_debug()`
- `verifyr2::FileComparator$vrf_add_debug_files()`
- `verifyr2::FileComparator$vrf_close_debug()`
- `verifyr2::FileComparator$vrf_details()`
- `verifyr2::FileComparator$vrf_open_debug()`
- `verifyr2::FileComparator$vrf_option_value()`
- `verifyr2::FileComparator$vrf_summary()`
- `verifyr2::TxtFileComparator$vrf_contents_inner()`
- `verifyr2::TxtFileComparator$vrf_details_inner()`
- `verifyr2::TxtFileComparator$vrf_summary_inner()`

}} \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-PdfFileComparator-vrf_contents}{}}} \subsection{Method \code{vrf_contents()}}{ Method for getting the single file contents for the comparison. The method returns the file contents in two separate vectors inside a list. The first vector is the file contents and the second one is the file contents with the rows matching the omit string excluded. This method can be overwritten by more specialized comparator classes. This method is intended to be called only by the comparator classes in the processing and shouldn't be called directly by the user. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{PdfFileComparator$vrf_contents(file, config, omit)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{file}}{file for which to get the contents} \item{\code{config}}{configuration values} \item{\code{omit}}{string pattern to omit from the comparison} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-PdfFileComparator-vrf_details_supported}{}}} \subsection{Method \code{vrf_details_supported()}}{ Inherited method for indicating whether detailed comparison is available with the current comparator. Returns an empty string if the comparator is is supported, otherwise a string that will be concatenated with the summary string. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{PdfFileComparator$vrf_details_supported(config)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{config}}{configuration values} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-PdfFileComparator-clone}{}}} \subsection{Method \code{clone()}}{ The objects of this class are cloneable with this method. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{PdfFileComparator$clone(deep = FALSE)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{deep}}{Whether to make a deep clone.} } \if{html}{\out{
}} } } }