% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_example.R
\name{run_example}
\alias{run_example}
\title{Call for shiny example where the user can test verifyr2 package functions}
\usage{
run_example(debug = FALSE)
}
\arguments{
\item{debug}{option to override debug configuration (TRUE only)}
}
\description{
\code{verifyr2::run_example} returns simple Shiny App where user can see how
the verifyr2 functions work
}
